/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/

#ifndef IDIOSKOPOSINTROSPECTABLE_H
#define IDIOSKOPOSINTROSPECTABLE_H

#include <map>
#include <string>

#include <sigc++/sigc++.h>

#include <idioskopos/pointer.h>
#include <idioskopos/error.h>
#include <idioskopos/object.h>
#include <idioskopos/property.h>
#include <idioskopos/referenceproperty.h>

namespace Idioskopos
  {

  typedef std::map<std::string, Object::pointer> DynamicProperties;
  typedef std::map<std::string, Object*> Properties;

  /**
   * @author Rick L Vinyard Jr
   */
  class Introspectable: public Object
    {
    public:
      typedef IdioskoposPointer<Introspectable> pointer;

      Introspectable();
      Introspectable( Introspectable& container, const std::string& name );

      ~Introspectable();

      Object* get_property( const std::string& name );

      bool is_property( const std::string& name );

      /**
       * Adds the given property that will be identified by the given name.
       */
      bool add_property( const std::string& name, Object& property );

      template <typename T>
      PropertyBase<T>* create_property( const std::string& name, const T& value = T() );

      template <typename T>
      PropertyBase<T>* create_property_reference( const std::string& name, T& value );

      void remove_property( const std::string& name );

      sigc::signal<void, const std::string&> signal_property_changed();

      template <typename T>
      PropertyBase<T>& property( const std::string& name );
      template <typename T>
      PropertyBase<T>* property_ptr( const std::string& name );

      virtual std::string get_class_name()
      {
        return "Introspectable";
      }

      const std::type_info& type() const
        {
          return typeid( Introspectable );
        }

      /**
       * Default parent implementation returns 0, so Introspectable must
       * provide a new implementation that will return the true number
       * of children.
       */
      virtual size_t properties_size();

      virtual size_t size() const
        {
          return 0;
        };

      virtual std::string xmlstring();

      virtual iterator properties_begin();
      virtual iterator properties_end();

      virtual Object& operator[] ( const std::string& name ) throw ( error::bad_property_name );

    protected:
      Properties m_properties;
      DynamicProperties m_created_properties;

      std::string m_separator;
      sigc::signal<void, const std::string&> m_signal_property_changed;
      std::map<Object*, sigc::connection> m_property_changed_connection;

      void on_property_changed( const std::string& propname );

      virtual iterator next_property( const iterator& i );
      virtual iterator previous_property( const iterator& i );


    };

  template <typename T>
  inline
  PropertyBase<T>& Introspectable::property( const std::string& name )
  {
    Object * pb = get_property( name );
    if ( pb == NULL )
      throw error::bad_property_name();
    try
      {
        PropertyBase<T>& prop = dynamic_cast<PropertyBase<T>&>( *pb );
        return prop;
      }
    catch ( ... )
      {
        throw error::cast::wrong_property_type();
      }
  }

  template <typename T>
  inline
  PropertyBase<T>* Introspectable::property_ptr( const std::string& name )
  {
    PropertyBase<T>* prop;
    Object* pb = get_property( name );
    if ( pb == NULL )
      return NULL;
    try
      {
        prop = dynamic_cast<PropertyBase<T>*>( pb );
        return prop;
      }
    catch ( ... )
      {
        return NULL;
      }
  }

  template <typename T>
  inline
  PropertyBase<T>* Introspectable::create_property( const std::string& name, const T& value )
  {
    if ( is_property( name ) )
      return NULL;

    Object::pointer sp( new Property<T>( value ) );
    m_properties[ name ] = sp.get();
    m_created_properties[ name ] = sp;
    sp->signal_changed().connect( sigc::bind( sigc::mem_fun( *this, &Introspectable::on_property_changed ), name ) );
    return dynamic_cast<PropertyBase<T>*>( sp.get() );
  }

  template <typename T>
  inline
  PropertyBase<T>* Introspectable::create_property_reference( const std::string& name, T& value )
  {
    if ( is_property( name ) )
      return NULL;

    Object::pointer sp( new ReferenceProperty<T>( value ) );
    m_properties[ name ] = sp.get();
    m_created_properties[ name ] = sp;
    sp->signal_changed().connect( sigc::bind( sigc::mem_fun( *this, &Introspectable::on_property_changed ), name ) );
    return dynamic_cast<PropertyBase<T>*>( sp.get() );
  }


}

#endif
