/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/

#ifndef IDIOSKOPOSOBJECT_H
#define IDIOSKOPOSOBJECT_H

#include <string>
#include <iostream>
#include <sigc++/sigc++.h>
#include <typeinfo>

#include <idioskopos/pointer.h>
#include <idioskopos/error.h>

namespace Idioskopos
  {

  class Introspectable;

  /**
   * Serves as a common interface to all properties.
   *
   * Namely, it provides an top-level interface for signals and RTTI information.
   * Since Property is a template class that inherits from Object, this
   * serves as a common ancestor.
   *
   * It is useful for generic containers of properties, such as that used
   * by Introspectable.
   *
   * @author Rick L Vinyard Jr
   */
  class Object: public virtual sigc::trackable
    {
    public:
      typedef IdioskoposPointer<Object> pointer;

      Object();

      /**
       * Constructor that adds this property to the introspectable container
       * and will identify this property by the given name.
       */
      Object(Introspectable& container, const std::string name);

      virtual ~Object();

      /**
       * Signal emitted when the property is changed.
       */
      sigc::signal<void>& signal_changed();

      /**
       * Pure virtual method implemented by children to return actual RTTI information from child.
       */
      virtual const std::type_info& type() const = 0;

      /**
       * Pure virtual method implemented by children to return an XML string containing
       * both type and value information.
       */
      virtual std::string xmlstring() = 0;

      /**
       * Virtual method implemented by children to return a string representation of
       * the contained value.
       */
      virtual std::string valuestring()
      {
        return std::string();
      }

      /**
       * Pure virtual method implemented by children to return a 'meaningful'
       * byte size value. In the case of the Property template it returns the
       * size in bytes of the contained value.
       */
      virtual size_t size() const = 0;

      /**
       * If this is a property container, this should be reimplemented
       * to provide a true count of the number of child properties this
       * container has.
       */
      virtual size_t properties_size()
      {
        return 0;
      }

    class iterator: public std::iterator<std::bidirectional_iterator_tag, Object>
        {
        public:
          typedef std::pair<std::string,Object*> ValuePair;

          iterator(Object* c=NULL, const ValuePair& value=ValuePair("",NULL));

          ~iterator();

          iterator& operator=(const iterator& other);

          bool operator==(const iterator& other) const;

          bool operator!=(const iterator& other) const;

          iterator& operator++();

          iterator operator++(int);

          iterator& operator--();

          iterator operator--(int);

          ValuePair& operator*();

          ValuePair* operator->();

        private:
          Object* m_container;
          ValuePair m_value;
        };

      virtual iterator properties_begin()
      {
        return properties_end();
      }
      virtual iterator properties_end()
      {
        return iterator();
      }

      virtual Object& operator[](const std::string& name) throw(error::bad_property_name)
      {
        throw error::bad_property_name();
      }

    protected:
      /**
       * Signal emitted when underlying data is changed.
       */
      sigc::signal<void> m_signal_changed;

      virtual iterator next_property(const iterator& i)
      {
        return iterator(this, iterator::ValuePair("",NULL));
      }

      virtual iterator previous_property(const iterator& i)
      {
        return iterator(this, iterator::ValuePair("",NULL));
      }

    };

}

#endif
