/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/

#ifndef IDIOSKOPOSPROPERTY_H
#define IDIOSKOPOSPROPERTY_H

#include <idioskopos/propertybase.h>

namespace Idioskopos {

  template <typename T>
  class ReadOnlyProperty: public ReadOnlyPropertyBase<T> {
    public:
      typedef IdioskoposPointer<ReadOnlyProperty> pointer;
      typedef T value_type;

      /**
      * Default constructor that doesn't do much.
       */
      ReadOnlyProperty () { }

      /**
       * Initializes the contained value to the parameter's value.
       */
      ReadOnlyProperty ( const T& val ) : m_value( val ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name.
       */
      ReadOnlyProperty( Introspectable& container, const std::string& name ) : ReadOnlyPropertyBase<T>( container, name ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name, and initializes the contained value to the value of
       * the third parameter.
       */
      ReadOnlyProperty ( Introspectable& container, const std::string& name, const T& val ) : ReadOnlyPropertyBase<T>( container, name ), m_value( val ) { }

      virtual ~ReadOnlyProperty() { }

      /** The canonical get() accessor method. */
      virtual T get
        () const {
          return m_value;
        }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return & m_value;
      }

    protected:
      T m_value;

  };

  template <typename T>
  class WriteOnlyProperty: public WriteOnlyPropertyBase<T> {
    public:
      typedef IdioskoposPointer<WriteOnlyProperty> pointer;
      typedef T value_type;

      /**
           * Default constructor that doesn't do much.
       */
      WriteOnlyProperty () { }

      /**
           * Initializes the contained value to the parameter's value.
       */
      WriteOnlyProperty ( const T& val ) : m_value( val ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name.
       */
      WriteOnlyProperty( Introspectable& container, const std::string& name ) : WriteOnlyPropertyBase<T>( container, name ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name, and initializes the contained value to the value of
           * the third parameter.
       */
      WriteOnlyProperty ( Introspectable& container, const std::string& name, const T& val ) : WriteOnlyPropertyBase<T>( container, name ), m_value( val ) { }

      virtual ~WriteOnlyProperty() { }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set
        ( const T& val ) {
        m_value = val;
      }

    protected:
      T m_value;

  };

  template <typename T>
  class Property: public PropertyBase<T> {
    public:
      typedef IdioskoposPointer<Property> pointer;
      typedef T value_type;

      /**
           * Default constructor that doesn't do much.
       */
      Property () { }

      /**
           * Initializes the contained value to the parameter's value.
       */
      Property ( const T& val ) : m_value( val ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name.
       */
      Property( Introspectable& container, const std::string& name ) : Object( container, name ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name, and initializes the contained value to the value of
           * the third parameter.
       */
      Property ( Introspectable& container, const std::string& name, const T& val ) : Object( container, name ), m_value( val ) { }

      virtual ~Property() { }

      /** The canonical get() accessor method. */
      virtual T get
        () const {
          return m_value;
        }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return & m_value;
      }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set
        ( const T& val ) {
        m_value = val;
      }

    protected:
      T m_value;

  };

}


#endif

