/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/

#ifndef IDIOSKOPOSPROPERTYBASE_H
#define IDIOSKOPOSPROPERTYBASE_H

#include <idioskopos/readonlypropertybase.h>
#include <idioskopos/writeonlypropertybase.h>

// TODO
// Need to make properties restrictable on the introspection point, with proxies to temporarily restrict or enhance permissions

namespace Idioskopos
  {

  /**
   * Implements a simple property interface with predefined get/set methods.
   *
   * A property is intended to encapsulate a simple value that is
   * most typically modified through assignments, i.e. integers,
   * doubles, strings, et. al.
   *
   * @author Rick L Vinyard Jr
  */
  template <typename T>
      class PropertyBase: public virtual ReadOnlyPropertyBase<T>, public virtual WriteOnlyPropertyBase<T>
    {
    public:
      typedef IdioskoposPointer<PropertyBase> pointer;
      typedef T value_type;

      /**
       * Default constructor that doesn't do much.
       */
      PropertyBase () { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name.
       */
      PropertyBase(Introspectable& container, const std::string& name): Object(container, name)
      { }

      virtual ~PropertyBase()
      { }

      /**
      * Allows assignment to the property from the contained value type.
      */
      PropertyBase<T>& operator= (const T& val)
      {
        this->set(val);
        return *this;
      }


      /**
       * Function call syntax
       *
       * Set the value of the property with property(value) notation.
       * @return The new value of the property
       */
      PropertyBase<T>& operator() (const T& val)
      {
        this->set(val);
        return *this;
      }

      /**
       * Arithmetic assignment operator
       *
       * This method is implemented as a template for two reasons:
       *  - So that it will work with a variety of operands
       *  - So that a stub will not be generated if the operator is not defined for T
       *
       * T + X must be well defined
       */
      template <typename X>
      PropertyBase<T>& operator+=(X other)
      {
        this->set( this->get() + other );
        return *this;
      }

      /**
       * Arithmetic assignment operator
       *
       * This method is implemented as a template for two reasons:
       *  - So that it will work with a variety of operands
       *  - So that a stub will not be generated if the operator is not defined for T
       *
       * T - X must be well defined
       */
      template <typename X>
      PropertyBase<T>& operator-=(X other)
      {
        this->set( this->get() - other );
        return *this;
      }

      /**
       * Arithmetic assignment operator
       *
       * This method is implemented as a template for two reasons:
       *  - So that it will work with a variety of operands
       *  - So that a stub will not be generated if the operator is not defined for T
       *
       * T * X must be well defined
       */
      template <typename X>
      PropertyBase<T>& operator*=(X other)
      {
        this->set( this->get() * other );
        return *this;
      }

      /**
       * Arithmetic assignment operator
       *
       * This method is implemented as a template for two reasons:
       *  - So that it will work with a variety of operands
       *  - So that a stub will not be generated if the operator is not defined for T
       *
       * T / X must be well defined
       */
      template <typename X>
      PropertyBase<T>& operator/=(X other)
      {
        this->set( this->get() / other );
        return *this;
      }

      /**
       * Arithmetic assignment operator
       *
       * This method is implemented as a template for two reasons:
       *  - So that it will work with a variety of operands
       *  - So that a stub will not be generated if the operator is not defined for T
       *
       * T % X must be well defined
       */
      template <typename X>
      PropertyBase<T>& operator%=(X other)
      {
        this->set( this->get() % other );
        return *this;
      }

    };

}

#endif

