/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/

#ifndef IDIOSKOPOSREADONLYPROPERTYBASE_H
#define IDIOSKOPOSREADONLYPROPERTYBASE_H

#include <idioskopos/utility_stream.h>

namespace Idioskopos {

/**
 * Implements a simple read only property. Using this class ensures
 * that the value may not be changed through introspection methods.
 *
 * This is a pure virtual class. Children should override the get()
 * method to return the real value and the get_pointer() to return
 * a pointer to the real value.
 *
 * @author Rick L Vinyard Jr
 */
template <typename T>
  class ReadOnlyPropertyBase: public virtual Object {
  public:
    typedef IdioskoposPointer<ReadOnlyPropertyBase> pointer;
    typedef T value_type;

    /** Default constructor that doesn't do much. */
    ReadOnlyPropertyBase () { }

    /**
     * Makes this a property of the introspectable container accessible through
     * the provided name.
     */
  ReadOnlyPropertyBase(Introspectable& container, const std::string& name): Object(container, name) { }

    /** Destructor */
    virtual ~ReadOnlyPropertyBase() { }

    /** The canonical get() accessor method. */
    virtual T get() const = 0;

    /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
    virtual T* get_pointer() = 0;

    /** Allows for implicit casts to the contained type. */
    operator T () const {
      return this->get();
    }

    /**
     * Function call syntax
     *
     * Get the value of the property with property() notation.
     * @return the current value of the property
     */
    T& operator() () const {
      return this->get();
    }

    /**
     * A convenience operator when the property contains a
     * class.
     *
     * Allows class method to be accessed using notation:
     * property->class_method()
     */
    T* operator->() {
      return get_pointer();
    }

    /**
     * The get_xmlstring function is a template method with full
     * specializations for known types.
     * The default implementation returns an XML string filled
     * with the RTTI typeinfo id. This is not a portable approach.
     * However, if you provide a specialization of get_xmlstring()
     * for your class, it _will_ be a portable solution.
     */
    virtual std::string xmlstring() {
      return get_xmlstring( static_cast<T>(*this));
    }

  /**
   * Similar in concept to the xmlstring() method, this provides a
   * string representation of the contained value if the get_valuestring()
   * template has a meaningful template specialization.
   */
    virtual std::string valuestring() { return get_valuestring(static_cast<T>(*this)); }

        /**
     * Returns the RTTI type id of the contained type
         */
    const std::type_info& type() const {
      return typeid(T);
    }

    /**
     * Returns the size in bytes of the contained type,
     * and not necessarily the size of this class.
     */
    size_t size() const {
      return sizeof(T);
    }

  };

}

#endif

