/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/

#ifndef IDIOSKOPOSREFERENCEPROPERTY_H
#define IDIOSKOPOSREFERENCEPROPERTY_H

#include <idioskopos/object.h>

namespace Idioskopos {

  template <typename T>
  class ReadOnlyReferenceProperty: public ReadOnlyProperty<T> {
    public:
      typedef IdioskoposPointer<ReadOnlyReferenceProperty> pointer;
      typedef T value_type;

      /**
       * Initializes the contained value to the parameter's value.
       */
      ReadOnlyReferenceProperty ( T& val ) : m_value( val ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name, and initializes the contained value to the value of
       * the third parameter.
       */
      ReadOnlyReferenceProperty ( Introspectable& container, const std::string& name, T& val ) : ReadOnlyProperty<T>( container, name ), m_value( val ) { }

      virtual ~ReadOnlyReferenceProperty() { }

      /** The canonical get() accessor method. */
      virtual T get
        () const {
          return m_value;
        }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return & m_value;
      }

    protected:
      T& m_value;

  };

  template <typename T>
  class WriteOnlyReferenceProperty: public WriteOnlyProperty<T> {
    public:
      typedef IdioskoposPointer<WriteOnlyReferenceProperty> pointer;
      typedef T value_type;

      /**
           * Initializes the contained value to the parameter's value.
       */
      WriteOnlyReferenceProperty ( T& val ) : m_value( val ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name, and initializes the contained value to the value of
           * the third parameter.
       */
      WriteOnlyReferenceProperty ( Introspectable& container, const std::string& name, T& val ) : WriteOnlyProperty<T>( container, name ), m_value( val ) { }

      virtual ~WriteOnlyReferenceProperty() { }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set
        ( const T& val ) {
        m_value = val;
      }

    protected:
      T& m_value;

  };

  template <typename T>
  class ReferenceProperty: public Property<T> {
    public:
      typedef IdioskoposPointer<ReferenceProperty> pointer;
      typedef T value_type;

      /**
           * Initializes the contained value to the parameter's value.
       */
      ReferenceProperty ( T& val ) : m_value( val ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name, and initializes the contained value to the value of
           * the third parameter.
       */
      ReferenceProperty ( Introspectable& container, const std::string& name, T& val ) : Object( container, name ), m_value( val ) { }

      virtual ~ReferenceProperty() { }

      /** The canonical get() accessor method. */
      virtual T get
        () const {
          return m_value;
        }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return & m_value;
      }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set
        ( const T& val ) {
        m_value = val;
      }

    protected:
      T& m_value;

  };

}


#endif

