/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#ifndef IDIOSKOPOSUTILITY_STREAM_H
#define IDIOSKOPOSUTILITY_STREAM_H

#include <sstream>

namespace Idioskopos {
/**
 * This template function returns an xml string of the property
 * and the contained value. Since the default template returns
 * a value for type that is filled with RTTI information this
 * is definiately not a portable approach. However, if a full
 * specialization is provided, it becomes a portable solution
 * for those types that have template specializations.
 *
 * The macro IDIOSKOPOS_XMLSTRING is provided to make creating
 * template specializations easier. The first parameter to the
 * macro is your type, and the second parameter is the typeid
 * string.
 *
 * The macro IDIOSKOPOS_STREAMABLE creates template specializations
 * for both the get_xmlstring() and get_valuestring() template functions.
 */
template <typename T>
  inline std::string get_xmlstring(const T& t) {
    std::ostringstream out;
    out << "< value type=\""
      << typeid(T).name() << "\"/>";
    return out.str();
  }


/**
 * A convenience macro for that defines a template specialization for
 * the get_xmlstring() function template, associating a given string
 * with a type.
 */
#define IDIOSKOPOS_XMLSTRING( Type, TypeString )                           \
inline std::string get_xmlstring(const Type& t) {   \
  std::ostringstream out;                                                \
  out << "< value type=\""                                             \
  << TypeString << "\">"                                             \
  << t << "</value>";                   \
  return out.str();                                                      \
  }

template<typename T>
  inline std::string get_valuestring(const T& t) {
    return std::string();
  }

/**
 * A convenience macro for that defines a template specialization for
 * the get_valuestring() function template.
 */
#define IDIOSKOPOS_VALUESTRING( Type )                                      \
inline std::string get_valuestring(const Type& t) {  \
  std::ostringstream out;                                                   \
  out << t;                                       \
  return out.str();                                                         \
  }

/**
 * A convenience macro for that defines template specializations for
 * both get_xmlstring() and get_valuestring() function templates.
 */
#define IDIOSKOPOS_STREAMABLE( Type, TypeString )   \
IDIOSKOPOS_XMLSTRING( Type, TypeString )           \
  IDIOSKOPOS_VALUESTRING(Type )

  IDIOSKOPOS_STREAMABLE(float,"float");
IDIOSKOPOS_STREAMABLE(double,"double");
IDIOSKOPOS_STREAMABLE(long double,"long double");
IDIOSKOPOS_STREAMABLE(short int,"short int");
IDIOSKOPOS_STREAMABLE(unsigned short int,"unsigned short int");
IDIOSKOPOS_STREAMABLE(int,"int");
IDIOSKOPOS_STREAMABLE(unsigned int,"unsigned int");
IDIOSKOPOS_STREAMABLE(long int,"long int");
IDIOSKOPOS_STREAMABLE(unsigned long int,"unsigned long int");
IDIOSKOPOS_STREAMABLE(char,"char");
IDIOSKOPOS_STREAMABLE(unsigned char,"unsigned char");
IDIOSKOPOS_STREAMABLE(bool,"bool");
IDIOSKOPOS_STREAMABLE(std::string,"std::string");
}

#endif

