/***************************************************************************
*   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/

#ifndef IDIOSKOPOSVIRTUALPROPERTY_H
#define IDIOSKOPOSVIRTUALPROPERTY_H

#include <idioskopos/object.h>

namespace Idioskopos {

  template <typename T>
  class ReadOnlyVirtualProperty: public ReadOnlyProperty<T> {
    public:
      typedef IdioskoposPointer<ReadOnlyVirtualProperty> pointer;
      typedef T value_type;
      typedef sigc::slot<T> GetSlotType;

      /**
      * Default constructor that doesn't do much.
       */
      ReadOnlyVirtualProperty ( sigc::slot<T> get_slot = GetSlotType() ) : m_get_slot( get_slot ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name, and initializes the contained value to the value of
       * the third parameter.
       */
      ReadOnlyVirtualProperty ( Introspectable& container, const std::string& name, sigc::slot<T> get_slot = GetSlotType() ) : ReadOnlyProperty<T>( container, name ), m_get_slot( get_slot ) { }

      virtual ~ReadOnlyVirtualProperty() { }

      /** The canonical get() accessor method. */
      virtual T get() const {
          if ( m_get_slot )
            return m_get_slot();
          else
            // TODO throw a proper exception class instead of a general throw
            throw;
        }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return 0x00;
      }

      sigc::slot<T> get_slot() { return m_get_slot; }

      void get_slot( sigc::slot<T> slot ) { m_get_slot = slot; }

      void get_slot_clear() { m_get_slot = GetSlotType(); }

    protected:
      sigc::slot<T> m_get_slot;

  };

  template <typename T>
  class WriteOnlyVirtualProperty: public WriteOnlyProperty<T> {
    public:
      typedef IdioskoposPointer<WriteOnlyVirtualProperty> pointer;
      typedef T value_type;
      typedef sigc::slot<void, T> SetSlotType;

      /**
           * Initializes the contained value to the parameter's value.
       */
      WriteOnlyVirtualProperty ( sigc::slot<void, T> set_slot = SetSlotType() ) : m_set_slot( set_slot ) { }

      /**
           * Makes this a property of the introspectable container accessible through
           * the provided name, and initializes the contained value to the value of
           * the third parameter.
       */
      WriteOnlyVirtualProperty ( Introspectable& container, const std::string& name, sigc::slot<void, T> set_slot = SetSlotType() ) : WriteOnlyProperty<T>( container, name ), m_set_slot( set_slot ) { }

      virtual ~WriteOnlyVirtualProperty() { }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set
        ( const T& val ) {
        if ( m_set_slot )
          m_set_slot( val );
      }

      sigc::slot<void, T> set_slot() { return m_set_slot; }

      void set_slot( sigc::slot<void, T> slot ) { m_set_slot = slot; }

      void set_slot_clear() { m_set_slot = SetSlotType(); }

    protected:
      sigc::slot<void, T> m_set_slot;

  };

  template <typename T>
  class VirtualProperty: public Property<T> {
    public:
      typedef IdioskoposPointer<VirtualProperty> pointer;
      typedef T value_type;
      typedef sigc::slot<T> GetSlotType;
      typedef sigc::slot<void, T> SetSlotType;

      /**
       * Default constructor that doesn't do much.
       */
      VirtualProperty ( ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name, and initializes the contained value to the value of
       * the third parameter.
       */
      VirtualProperty ( Introspectable& container, const std::string& name ) : Object( container, name ) { }

      /**
       * Default constructor that doesn't do much.
       */
      VirtualProperty ( sigc::slot<T> get_slot, sigc::slot<void, T> set_slot ) : m_get_slot( get_slot ), m_set_slot( set_slot ) { }

      /**
       * Makes this a property of the introspectable container accessible through
       * the provided name, and initializes the contained value to the value of
       * the third parameter.
       */
      VirtualProperty ( Introspectable& container, const std::string& name, sigc::slot<T> get_slot, sigc::slot<void, T> set_slot ) : Object( container, name ), m_get_slot( get_slot ), m_set_slot( set_slot ) { }

      virtual ~VirtualProperty() { }

      /** The canonical get() accessor method. */
      virtual T get() const {
        if ( m_get_slot )
          return m_get_slot();
        else
            // TODO throw a proper exception class instead of a general throw
          throw;
      }

      /** Should return a pointer to the contained object, or NULL if pointers are not allowed */
      virtual T* get_pointer() {
        return 0x00;
      }

      /** The canonical set() accessor method that also emits the changed signal. */
      virtual void set( const T& val ) {
        if ( m_set_slot )
          m_set_slot( val );
      }

      sigc::slot<T> get_slot() { return m_get_slot; }

      void get_slot( sigc::slot<T> slot ) { m_get_slot = slot; }

      void get_slot_clear() { m_get_slot = GetSlotType(); }

      sigc::slot<void, T> set_slot() { return m_set_slot; }

      void set_slot( sigc::slot<void, T> slot ) { m_set_slot = slot; }

      void set_slot_clear() { m_set_slot = SetSlotType(); }

    protected:
      sigc::slot<T> m_get_slot;
      sigc::slot<void, T> m_set_slot;

  };


// Utility macros
  /**
   * This is a convenience macro to be used to create sigc slots
   * Note that this macro assumes the get and set methods are members of the current class
   * @param property The property to initialize
   * @param id The string id of the property
   * @param getter The get method to use
   * @param setter The set method to use
   */
#define INIT_VIRTUAL_PROPERTY( property, id, getter, setter ) \
  property( *this, id, sigc::mem_fun(*this, &getter), sigc::mem_fun(*this, &setter) )

/**
 * This convenience macro can be used to disambiguate an overloaded set method
 * @param class The class with the set methods
 * @param type The type of this property
 * @param property The property to initialize
 * @param id The string id of the property
 * @param getter The get method to use
 * @param setter The set method to use
 */
#define INIT_VIRTUAL_PROPERTY_OVERLOADED( class, type, property, id, getter, setter )   \
  property( *this, id, sigc::mem_fun(*this, &getter),                                   \
                       sigc::mem_fun(*this, static_cast<void(class::*)(type)>(&setter)) \
          )

}


#endif

